#ifndef MAIN_DEFINED
#define MAIN_DEFINED

/* global socket connection id */
extern int gbConnID;

/* Start string for communication */
#define START_STR  "Player\n"


/* Board height and width */
extern int gbWidth, gbHeight;

/* We allocate extra outer rows and columns, and fill them with walls */
#define MEM_WIDTH (gbWidth + 2)
#define MEM_HEIGHT (gbHeight + 2)


/* Max Buffer length */
#define MAX_LINE 1000000
#define MAX_LEN LONG_MAX


/* Tile types */
#define WALL '#'
#define WATER '~'
#define PLAIN '.'
#define HOME '@'


#define DANGER_BID  2 /* percent */
#define DROP_BID    1 /* percent */


/* Our configuration */
extern int gbMyID;
extern int gbMyCapacity;
extern int gbMyMoney;
extern int gbMyX;
extern int gbMyY;

/* Structure for list of integers */
typedef struct t_IntListNode
{
    struct t_IntListNode *next;
    int val;
}INTLISTNODE;

/* Robot */
typedef struct
{
    int id;
    int x, y;
    INTLISTNODE *packageIDs;
    int updated;
}ROBOT;

/* structure for list of robots */
typedef struct t_RbtListNode
{
    struct t_RbtListNode *next;
    ROBOT rbt;
}RBTLISTNODE;

/* Our current robot list */
extern RBTLISTNODE *gbRbts;


/* Package */
typedef struct
{
    int id;
    int x, y; //source
    int destx, desty;
    int weight;
}PACKAGE;

/* structure for list of packages */
typedef struct t_PkgListNode
{
    struct t_PkgListNode *next;
    PACKAGE pkg;
}PKGLISTNODE;

/* Our current package list */
extern PKGLISTNODE *gbMyPkgs;

/* List of other packages we know of */
extern PKGLISTNODE *gbKnownPkgs;

typedef struct
{
    char type;
    ROBOT *robot;
    int pathVal;
}TILE;

typedef struct t_HomeBaseNode
{
    struct t_HomeBaseNode *next;
    int x, y;
}HOMEBASENODE;

extern HOMEBASENODE *gbHomeBase;

/* Board */
extern TILE *gbBoard;

/* Board access macro */
#define Tile(x, y) (gbBoard[(y) * (MEM_WIDTH) + (x)])


/* Actions */
enum CmdType
{
    MOVE,
    PICK,
    DROP,
};

/* Directions */
#define NORTH 'N'
#define SOUTH 'S'
#define EAST 'E'
#define WEST 'W'

#define NODIR '%'


/* Function to do all the steps in the "Initialization Phase"
1. Connect and send START_STR
2. Read Board
3. Read config (our ID, capacity and money)
4. Read initial server response (location of other players)
*/
void DoInitPhase(char *server, int port);

/* Helper functions to be used by DoInitPhase above */
void ReadBoard();
void ReadConfig();
void ReadSrvrResponse();
PKGLISTNODE *ReadAvailablePkgs();
/* (ReadSrvrResponse will also be used to read the response at each turn) */

/* At each turn */
PKGLISTNODE * ReadAvailablePkgs();
void IssueMove(long bid, char direction);
void IssuePick(long bid, INTLISTNODE *pkgIDs);
void IssueDrop(long bid, INTLISTNODE *pkgIDs);

/* List helper functions */

/*    Integer list */
INTLISTNODE *NewIntNode(int val);
INTLISTNODE *AddIntNode(INTLISTNODE *head, INTLISTNODE *newNode);
INTLISTNODE *DeleteIntNode(INTLISTNODE *head, int val);
void FreeIntList(INTLISTNODE *head);
INTLISTNODE *FindIntNode(INTLISTNODE *head, int val);
int IntListLen(INTLISTNODE *head);

/*   Package list */
PKGLISTNODE *NewPkgNode(const PACKAGE *pkg);
PKGLISTNODE *AddPkgNode(PKGLISTNODE *head, PKGLISTNODE *newNode);
PKGLISTNODE *DeletePkgNode(PKGLISTNODE *head, int pkgID);
void FreePkgListNode(PKGLISTNODE *head);
int  PkgListLen(PKGLISTNODE *head);
int PkgListWtSum(PKGLISTNODE *head);
PKGLISTNODE *FindPkgNode(PKGLISTNODE *head, int pkgID);

/* Robot list */
RBTLISTNODE *NewRbtNode(const ROBOT *rbt);
RBTLISTNODE *AddRbtNode(RBTLISTNODE *head, RBTLISTNODE *newNode);
RBTLISTNODE *DeleteRbtNode(RBTLISTNODE *head, int rbtID);
void FreeRbtListNode(RBTLISTNODE *head);
RBTLISTNODE *FindRbtNode(RBTLISTNODE *head, int rbtID);


/* HomeBase list */
HOMEBASENODE *NewHomeBaseNode(int x, int y);
HOMEBASENODE *AddHomeBaseNode(HOMEBASENODE *head, HOMEBASENODE *newNode);
HOMEBASENODE *DeleteHomeBaseNode(HOMEBASENODE *head, HOMEBASENODE *thisNode);

/* Function from Socket library

     This function gets the next line of text from the server.
     The contents of the buffer are valid only till the next
     call to GetNextLine.
*/
char *GetNextLine();

/* SendALine sends a line */
void SendALine(char *line);

/* Functions to implement game strategy

ShortestPath must use the game board, and consider all obstacles
like water/wall etc. Use Djikstra's algo.

*length should be -1 if there is no path
*nextMoveDir is the direction for the next move - one of NORTH, SOUTH, EAST, WEST.
*/

void ShortestPath (int xOrigin, int yOrigin );
void GetBidValue(long *bid, int *timeToDrop);
void FindMinNeighbour(char *dir);
void FindAnyNeighbour(char *dir);

#endif

